@@ -54,24 +54,24 @@ handle_glm_prior <- function(prior, nvars, default_scale, link,
   if (!length(prior))
     return(list(prior_dist = 0L, prior_mean = as.array(rep(0, nvars)),
                 prior_scale = as.array(rep(1, nvars)),
-                prior_df = as.array(rep(1, nvars))))
+                prior_df = as.array(rep(1, nvars)), prior_dist_name = NA))
   if (!is.list(prior)) 
     stop(sQuote(deparse(substitute(prior))), " should be a named list")
   
-  prior_dist <- prior$dist
+  prior_dist_name <- prior$dist
   prior_scale <- prior$scale
   prior_mean <- prior$location
   prior_df <- prior$df
   prior_df[is.na(prior_df)] <- 1
-  if (!prior_dist %in% unlist(ok_dists)) {
+  if (!prior_dist_name %in% unlist(ok_dists)) {
     stop("The prior distribution should be one of ",
          paste(names(ok_dists), collapse = ", "))
-  } else if (prior_dist %in% c("normal", "t", "cauchy")) {
-    prior_dist <- ifelse(prior_dist == "normal", 1L, 2L)
+  } else if (prior_dist_name %in% c("normal", "t", "cauchy")) {
+    prior_dist <- ifelse(prior_dist_name == "normal", 1L, 2L)
     prior_scale <- set_prior_scale(prior_scale, default = default_scale, 
                                    link = link)
   } else {
-    prior_dist <- ifelse(prior_dist == "hs", 3L, 4L)
+    prior_dist <- ifelse(prior_dist_name == "hs", 3L, 4L)
   }
   
   prior_df <- maybe_broadcast(prior_df, nvars)
@@ -79,5 +79,5 @@ handle_glm_prior <- function(prior, nvars, default_scale, link,
   prior_mean <- maybe_broadcast(prior_mean, nvars)
   prior_mean <- as.array(prior_mean)
   prior_scale <- maybe_broadcast(prior_scale, nvars)
-  return(nlist(prior_dist, prior_mean, prior_scale, prior_df)) 
+  return(nlist(prior_dist, prior_mean, prior_scale, prior_df, prior_dist_name))
 }
