@@ -128,6 +128,7 @@ used.variational <- function(x) {
 #
 # @param x A stanreg object.
 is.mer <- function(x) {
+  stopifnot(is.stanreg(x))
   check1 <- inherits(x, "lmerMod")
   check2 <- !is.null(x$glmod)
   if (check1 && !check2) {
@@ -469,31 +470,6 @@ set_prior_scale <- function(scale, default, link) {
   return(scale)
 }
 
-# Make prior.info list
-# @param user_call The user's call, i.e. match.call(expand.dots = TRUE).
-# @param function_formals Formal arguments of the stan_* function, i.e.
-#   formals().
-# @return A list containing information about the prior distributions and
-#   options used.
-get_prior_info <- function(user_call, function_formals) {
-  user <- grep("prior", names(user_call), value = TRUE)
-  default <- setdiff(grep("prior", names(function_formals), value = TRUE), 
-                     user)
-  U <- length(user)
-  D <- length(default)
-  priors <- list()
-  for (j in 1:(U + D)) {
-    if (j <= U) {
-      priors[[user[j]]] <- try(eval(user_call[[user[j]]]), silent = TRUE)
-    } else {
-      priors[[default[j-U]]] <- try(eval(function_formals[[default[j-U]]]), 
-                                    silent = TRUE)
-    } 
-  }
-  
-  return(priors)
-}
-
 
 # Methods for creating linear predictor
 #
@@ -558,7 +534,14 @@ get_z.lmerMod <- function(object) {
   Zt <- object$glmod$reTrms$Zt %ORifNULL% stop("Z not found")
   t(Zt)
 }
-
+#' @export
+get_z.gamm4 <- function(object) {
+  X <- get_x(object)
+  XZ <- object$x
+  Z <- XZ[,-c(1:ncol(X)), drop = FALSE]
+  Z <- Z[, !grepl("_NEW_", colnames(Z), fixed = TRUE), drop = FALSE]
+  return(Z)
+}
 
 # Get inverse link function
 #
@@ -632,3 +615,29 @@ check_reTrms <- function(reTrms) {
   }
   return(invisible(NULL))
 }
+
+#' @importFrom lme4 glmerControl
+make_glmerControl <- function(...) {
+  glmerControl(check.nlev.gtreq.5 = "ignore",
+               check.nlev.gtr.1 = "stop",
+               check.nobs.vs.rankZ = "ignore",
+               check.nobs.vs.nlev = "ignore",
+               check.nobs.vs.nRE = "ignore", ...)  
+}
+
+# Check if a fitted model (stanreg object) has weights
+# 
+# @param x stanreg object
+# @return Logical. Only TRUE if x$weights has positive length and the elements
+#   of x$weights are not all the same.
+#
+model_has_weights <- function(x) {
+  wts <- x[["weights"]]
+  if (!length(wts)) {
+    FALSE
+  } else if (all(wts == wts[1])) {
+    FALSE
+  } else {
+    TRUE
+  }
+}
