@@ -76,6 +76,9 @@ stan_biglm.fit <- function(b, R, SSR, N, xbar, ybar, s_y, has_intercept = TRUE,
     prior_scale_for_intercept <- prior_intercept$scale
     if (is.null(prior_scale_for_intercept))
       prior_scale_for_intercept <- 0
+    
+    # also add scale back to prior_intercept to pass to summarize_lm_prior later
+    prior_intercept$scale <- prior_scale_for_intercept
   }
   dim(R_inv) <- c(J, dim(R_inv))
   
@@ -113,5 +116,33 @@ stan_biglm.fit <- function(b, R, SSR, N, xbar, ybar, s_y, has_intercept = TRUE,
                  if (prior_PD == 0) "log-fit_ratio", 
                  "R2", "mean_PPD", "log-posterior")
   stanfit@sim$fnames_oi <- new_names
-  return(stanfit)
+
+  prior_info <- summarize_lm_prior(prior, prior_intercept)
+  structure(stanfit, prior.info = prior_info)
+}
+
+
+# internal ----------------------------------------------------------------
+
+# Create "prior.info" attribute needed for prior_summary()
+#
+# @param prior, prior_intercept User's prior and prior_intercept specifications
+# @return A named list with elements 'prior' and 'prior_intercept' containing 
+#   the values needed for prior_summary
+summarize_lm_prior <- function(prior, prior_intercept) {
+  flat <- !length(prior)
+  flat_int <- !length(prior_intercept)
+  
+  list(
+    prior = list(
+      dist = ifelse(flat, NA, "R2"),
+      location = ifelse(flat, NA, prior$location),
+      what = ifelse(flat, NA, prior$what)
+    ), 
+    prior_intercept = list(
+      dist = ifelse(flat_int, NA, "normal"),
+      location = ifelse(flat_int, NA, prior_intercept$location),
+      scale = ifelse(flat_int, NA, prior_intercept$scale)
+    )
+  )
 }
