@@ -73,7 +73,7 @@
 #' if (!exists("example_model")) example(example_model) 
 #' print(example_model, digits = 1)
 #' 
-#' @importFrom lme4 glFormula glmerControl
+#' @importFrom lme4 glFormula
 #' @importFrom Matrix Matrix t cBind
 stan_glmer <- function(formula, data = NULL, family = gaussian, 
                        subset, weights, 
@@ -89,11 +89,7 @@ stan_glmer <- function(formula, data = NULL, family = gaussian,
   mc <- match.call(expand.dots = FALSE)
   family <- validate_family(family)
   mc[[1]] <- quote(lme4::glFormula)
-  mc$control <- glmerControl(check.nlev.gtreq.5 = "ignore",
-                             check.nlev.gtr.1 = "stop",
-                             check.nobs.vs.rankZ = "ignore",
-                             check.nobs.vs.nlev = "ignore",
-                             check.nobs.vs.nRE = "ignore")
+  mc$control <- make_glmerControl()
   mc$prior <- mc$prior_intercept <- mc$prior_covariance <- mc$prior_ops <-
     mc$prior_PD <- mc$algorithm <- mc$scale <- mc$concentration <- mc$shape <-
     mc$adapt_delta <- mc$... <- mc$QR <- mc$sparse <- NULL
@@ -109,6 +105,8 @@ stan_glmer <- function(formula, data = NULL, family = gaussian,
     prior <- list()
   if (is.null(prior_intercept)) 
     prior_intercept <- list()
+  if (is.null(prior_covariance))
+    stop("'prior_covariance' can't be NULL.", call. = FALSE)
   if (!length(prior_ops)) 
     prior_ops <- list(scaled = FALSE, prior_scale_for_dispersion = Inf)
   group <- glmod$reTrms
@@ -121,13 +119,13 @@ stan_glmer <- function(formula, data = NULL, family = gaussian,
                           algorithm = algorithm, adapt_delta = adapt_delta,
                           group = group, QR = QR, sparse = sparse, ...)
 
-  Z <- pad_reTrms(Z = t(group$Zt), cnms = group$cnms, 
+  Z <- pad_reTrms(Ztlist = group$Ztlist, cnms = group$cnms, 
                   flist = group$flist)$Z
   colnames(Z) <- b_names(names(stanfit), value = TRUE)
+  
   fit <- nlist(stanfit, family, formula, offset, weights, 
                x = if (getRversion() < "3.2.0") cBind(X, Z) else cbind2(X, Z), 
                y = y, data, call, terms = NULL, model = NULL, 
-               prior.info = get_prior_info(call, formals()),
                na.action, contrasts, algorithm, glmod)
   out <- stanreg(fit)
   class(out) <- c(class(out), "lmerMod")
@@ -135,6 +133,26 @@ stan_glmer <- function(formula, data = NULL, family = gaussian,
   return(out)
 }
 
+make_Sigma <- function(mat, cnms) {
+  useSc <- "sigma" %in% colnames(mat)
+  if (useSc) sc <- mat[,"sigma"]
+  else sc <- 1
+  theta <- mat[,grepl("^theta\\[", colnames(mat)), drop = FALSE]
+  nc <- vapply(cnms, FUN = length, FUN.VALUE = 1L)
+  nms <- names(cnms)
+  Sigma_list <- apply(theta, 1, FUN = mkVarCorr, 
+                      sc = 1, cnms = cnms, nc = nc, nms = nms)
+  add <- function(x) Reduce("+", x)
+  Sigma <- sapply(Sigma_list[[1]], simplify = FALSE, FUN = `*`, y = 0)
+  for (i in seq_along(Sigma)) {
+    Sigma[[i]] <- add(lapply(Sigma_list, FUN = function(x) x[[i]])) / length(Sigma_list)
+    attr(Sigma[[i]], "stddev") <- sqrt(diag(Sigma[[i]]))
+    attr(Sigma[[i]], "correlation") <- cov2cor(Sigma[[i]])
+  }
+  Sigma <- structure(Sigma, useSc = useSc, sc = mean(sc), class = "VarCorr.merMod")
+  return(Sigma)
+}
+
 #' @rdname stan_glmer
 #' @export
 stan_lmer <- function(formula,
@@ -160,7 +178,7 @@ stan_lmer <- function(formula,
     names(call)[2L] <- "formula"
   mc[[1L]] <- quote(stan_glmer)
   mc$REML <- NULL
-  mc$family <- gaussian
+  mc$family <- "gaussian"
   out <- eval(mc, parent.frame())
   out$call <- call
   return(out)
