@@ -17,9 +17,10 @@
 
 #' Methods for stanreg objects
 #' 
-#' S3 methods for \link[=stanreg-objects]{stanreg} objects. There are also 
-#' several methods (listed in See Also, below) with their own individual help
-#' pages.
+#' The methods documented on this page are actually some of the least important 
+#' methods defined for \link[=stanreg-objects]{stanreg} objects. The most 
+#' important methods are documented separately, each with its own page. Links to
+#' those pages are provided in the \strong{See Also} section, below.
 #' 
 #' @name stanreg-methods
 #' @aliases VarCorr fixef ranef ngrps sigma
@@ -29,28 +30,11 @@
 #' @param ... Ignored, except by the \code{update} method. See
 #'   \code{\link{update}}.
 #' 
-#' @details Most of these methods are similar to the methods defined for objects
-#'   of class 'lm', 'glm', 'glmer', etc. However there are a few exceptions:
+#' @details The methods documented on this page are similar to the methods 
+#'   defined for objects of class 'lm', 'glm', 'glmer', etc. However there are a
+#'   few key differences:
 #'   
 #' \describe{
-#' \item{\code{confint}}{
-#' For models fit using optimization, confidence intervals are returned via a
-#' call to \code{\link[stats]{confint.default}}. If \code{algorithm} is
-#' \code{"sampling"}, \code{"meanfield"}, or \code{"fullrank"}, the
-#' \code{\link{posterior_interval}} function should be used to compute Bayesian
-#' uncertainty intervals.
-#' }
-#' 
-#' \item{\code{log_lik}}{
-#' For models fit using MCMC only, the \code{log_lik}
-#' function returns the \eqn{S} by \eqn{N} pointwise log-likelihood matrix,
-#' where \eqn{S} is the size of the posterior sample and \eqn{N} is the number
-#' of data points. Note: we use \code{log_lik} rather than defining a
-#' \code{\link[stats]{logLik}} method because (in addition to the conceptual
-#' difference) the documentation for \code{logLik} states that the return value
-#' will be a single number, whereas we return a matrix.
-#' }
-#' 
 #' \item{\code{residuals}}{
 #' Residuals are \emph{always} of type \code{"response"} (not \code{"deviance"}
 #' residuals or any other type). However, in the case of \code{\link{stan_polr}}
@@ -66,17 +50,37 @@
 #' \code{\link{mad}}. See the \emph{Uncertainty estimates} section in
 #' \code{\link{print.stanreg}} for more details.
 #' }
+#' \item{\code{confint}}{
+#' For models fit using optimization, confidence intervals are returned via a 
+#' call to \code{\link[stats]{confint.default}}. If \code{algorithm} is 
+#' \code{"sampling"}, \code{"meanfield"}, or \code{"fullrank"}, the
+#' \code{confint} will throw an error because the
+#' \code{\link{posterior_interval}} function should be used to compute Bayesian 
+#' uncertainty intervals.
+#' }
 #' }
 #' 
-#' @seealso
-#' Other S3 methods for stanreg objects, which have separate documentation, 
-#' including \code{\link{as.matrix.stanreg}}, \code{\link{plot.stanreg}}, 
-#' \code{\link{predict.stanreg}}, \code{\link{print.stanreg}}, and
-#' \code{\link{summary.stanreg}}.
-#' 
-#' \code{\link{posterior_interval}} and \code{\link{posterior_predict}} for 
-#' alternatives to \code{confint} and \code{predict} for models fit using MCMC 
-#' or variational approximation.
+#' @seealso 
+#' \itemize{
+#'  \item The \code{\link[=print.stanreg]{print}},
+#'    \code{\link[=summary.stanreg]{summary}}, and \code{\link{prior_summary}} 
+#'    methods for stanreg objects for information on the fitted model.
+#'  \item \code{\link{launch_shinystan}} to use the ShinyStan GUI to explore a
+#'    fitted \pkg{rstanarm} model.
+#'  \item The \code{\link[=plot.stanreg]{plot}} method to plot estimates and
+#'    diagnostics.
+#'  \item The \code{\link{pp_check}} method for graphical posterior predictive
+#'    checking.
+#'  \item The \code{\link{posterior_predict}} and \code{\link{predictive_error}}
+#'    methods for predictions and predictive errors.
+#'  \item The \code{\link{posterior_interval}} and \code{\link{predictive_interval}}
+#'    methods for uncertainty intervals for model parameters and predictions.
+#'  \item The \code{\link[=loo.stanreg]{loo}}, \code{\link{kfold}}, and
+#'  \code{\link{log_lik}} methods for leave-one-out or K-fold cross-validation, 
+#'    model comparison, and computing the log-likelihood of (possibly new) data.
+#'  \item The \code{\link[=as.matrix.stanreg]{as.matrix}}, \code{as.data.frame}, 
+#'    and \code{as.array} methods to access posterior draws.
+#' }
 #' 
 NULL
 
@@ -111,45 +115,6 @@ fitted.stanreg <- function(object, ...)  {
   object$fitted.values
 }
 
-#' Extract pointwise log-likelihood matrix
-#' 
-#' @export
-#' @keywords internal
-#' @param object Fitted model object.
-#' @param ... Arguments to methods. For example the
-#'   \code{\link[=stanreg-methods]{stanreg}} method accepts the argument
-#'   \code{newdata}.
-#' @return Pointwise log-likelihood matrix.
-#' @seealso \code{\link{log_lik.stanreg}}
-#' 
-log_lik <- function(object, ...) UseMethod("log_lik")
-
-#' @rdname stanreg-methods
-#' @export
-#' @param newdata For \code{log_lik}, an optional data frame of new data (e.g. 
-#'   holdout data) to use when evaluating the log-likelihood. See the 
-#'   description of \code{newdata} for \code{\link{posterior_predict}}.
-#' @param offset For \code{log_lik}, a vector of offsets. Only required if
-#'   \code{newdata} is specified and an \code{offset} was specified when fitting
-#'   the model.
-#' 
-log_lik.stanreg <- function(object, newdata = NULL, offset = NULL, ...) {
-  if (!used.sampling(object)) 
-    STOP_sampling_only("Pointwise log-likelihood matrix")
-  if (!is.null(newdata)) {
-    if ("gam" %in% names(object))
-      stop("'log_lik' with 'newdata' not yet supported ", 
-           "for models estimated via 'stan_gamm4'.")
-    newdata <- as.data.frame(newdata)
-  }
-  fun <- ll_fun(object)
-  args <- ll_args(object, newdata = newdata, offset = offset)
-  sapply(seq_len(args$N), function(i) {
-    as.vector(fun(i = i, data = args$data[i, , drop = FALSE], 
-                  draws = args$draws))
-  })
-}
-
 #' @rdname stanreg-methods
 #' @export 
 nobs.stanreg <- function(object, ...) {
@@ -343,20 +308,32 @@ sigma.stanreg <- function(object, ...) {
 #' @export
 #' @export VarCorr
 #' @importFrom nlme VarCorr
-#' @importFrom lme4 mkVarCorr
+#' @importFrom stats cov2cor
 VarCorr.stanreg <- function(x, sigma = 1, ...) {
+  mat <- as.matrix(x)
   cnms <- .cnms(x)
-  means <- get_posterior_mean(x$stanfit)
-  means <- means[, ncol(means)]
-  theta <- means[grepl("^theta_L", names(means))]
-  sc <- sigma.stanreg(x)
-  out <- lme4::mkVarCorr(sc = sc, cnms = cnms, 
-                         nc = vapply(cnms, FUN = length, FUN.VALUE = 1L),
-                         theta = theta / sc, nms = names(cnms))
-  structure(out, useSc = sc != 1, class = "VarCorr.merMod")
+  useSc <- "sigma" %in% colnames(mat)
+  if (useSc) sc <- mat[,"sigma"]
+  else sc <- 1
+  Sigma <- colMeans(mat[,grepl("^Sigma\\[", colnames(mat)), drop = FALSE])
+  nc <- vapply(cnms, FUN = length, FUN.VALUE = 1L)
+  nms <- names(cnms)
+  ncseq <- seq_along(nc)
+  spt <- split(Sigma, rep.int(ncseq, (nc * (nc + 1)) / 2))
+  ans <- lapply(ncseq, function(i) {
+    Sigma <- matrix(0, nc[i], nc[i])
+    Sigma[lower.tri(Sigma, diag = TRUE)] <- spt[[i]]
+    Sigma <- Sigma + t(Sigma)
+    diag(Sigma) <- diag(Sigma) / 2
+    rownames(Sigma) <- colnames(Sigma) <- cnms[[i]]
+    stddev <- sqrt(diag(Sigma))
+    corr <- cov2cor(Sigma)
+    structure(Sigma, stddev = stddev, correlation = corr)
+  })
+  names(ans) <- nms
+  structure(ans, sc = mean(sc), useSc = useSc, class = "VarCorr.merMod")
 }
 
-
 # Exported but doc kept internal ----------------------------------------------
 
 #' family method for stanreg objects
@@ -409,6 +386,7 @@ model.matrix.stanreg <- function(object, ...) {
 #'   that both default to \code{FALSE}.
 #' 
 formula.stanreg <- function(x, ...) {
+  if (inherits(x, "gamm4")) return(x$formula)
   if (is.mer(x)) 
     return(formula_mer(x, ...))
   
