@@ -76,11 +76,16 @@ expect_linpred_equal <- function(object, tol = 0.1) {
                check.attributes = FALSE)
 }
 
+test_that("posterior_predict returns object with correct classes", {
+  expect_s3_class(posterior_predict(example_model), 
+                  c("ppd", "matrix"))
+})
+
 # Error messages ----------------------------------------------------------
 context("posterior_predict (error messages)")
 test_that("posterior_predict errors if not a stanreg object", {
-  expect_error(posterior_predict(example_model$stanfit), "not a stanreg object")
-  expect_error(posterior_predict(summary(example_model)), "not a stanreg object")
+  expect_error(posterior_predict(example_model$stanfit), "no applicable method")
+  expect_error(posterior_predict(summary(example_model)), "no applicable method")
 })
 test_that("posterior_predict errors if model fit using optimization", {
   fit1 <- stan_glm(mpg ~ wt + cyl + am, data = mtcars, algorithm = "optimizing", 
@@ -101,7 +106,6 @@ test_that("posterior_predict errors if draws > posterior sample size", {
                regexp = "'draws' should be <= posterior sample size")
 })
 
-
 # VB ----------------------------------------------------------------------
 context("posterior_predict ok for vb")
 test_that("errors for optimizing and silent for vb", {
@@ -199,8 +203,7 @@ test_that("stan_gamm4 returns expected result for sleepstudy example", {
   # expect_equal(dim(yrep1), c(nrow(as.data.frame(fit)), nobs(fit)))
   expect_silent(yrep2 <- posterior_predict(fit, draws = 1))
   # expect_equal(dim(yrep2), c(1, nobs(fit)))
-  expect_error(posterior_predict(fit, newdata = model.frame(fit$gam)), 
-               "not yet supported for models estimated via 'stan_gamm4'")
+  expect_silent(posterior_predict(fit, newdata = sleepstudy))
 })
 
 
@@ -244,8 +247,8 @@ test_that("compatible with stan_lmer with offset", {
                       prior = normal(0,1), iter = ITER, chains = CHAINS,
                       seed = SEED, refresh = REFRESH, offset = offs))
   
-  expect_warning(posterior_predict(fit, newdata = mtcars[1:2, ]), 
-                 "offset")
+  expect_warning(posterior_predict(fit, newdata = mtcars[1:2, ], offset = offs),
+                 "STATS")
   check_for_error(fit, offset = offs)
 })
 
